package com.huawei.ethiopia.sdk;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.databinding.DataBindingUtil;

import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.widget.Toast;

import com.google.gson.Gson;
import com.google.gson.JsonObject;
import com.huawei.ethiopia.pay.sdk.api.core.data.PayInfo;
import com.huawei.ethiopia.pay.sdk.api.core.utils.PaymentManager;
import com.huawei.ethiopia.sdk.data.Order;
import com.huawei.ethiopia.sdk.databinding.ActivityMainBinding;
import com.huawei.ethiopia.sdk.utils.Constants;
import com.huawei.ethiopia.sdk.utils.EncryptUtils;
import com.huawei.ethiopia.sdk.widget.LoadingDialog;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.MediaType;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;
import okhttp3.ResponseBody;
import okhttp3.logging.HttpLoggingInterceptor;
import okio.Buffer;
import okio.BufferedSource;

/**
 * 第三方需要去自己商城下单（商城和埃塞后台对接好），不要使用demo中的下单placeOrder方法
 * demo中下单是直接去埃塞后台下单（为了演示）,实际上SDK只需要appId、shortCode、receiveCode这几个字段
 */
public class MainActivity extends AppCompatActivity {
    private static final String TAG = "MainActivity";

    private ActivityMainBinding binding;
    private OrderInfo orderInfo;
    private LoadingDialog loadingDialog;
    private String timestamp;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = DataBindingUtil.setContentView(this, R.layout.activity_main);
        binding.setOnPay(view -> placeOrder());
        orderInfo = new OrderInfo();
        orderInfo.shortCode = BuildConfig.SHORT_CODE;
        orderInfo.appId = BuildConfig.APP_ID;
        binding.setOrderInfo(orderInfo);
    }

    private void pay() {
        final PayInfo payInfo = new PayInfo.Builder()
                .setAppId(orderInfo.appId)
                .setShortCode(orderInfo.shortCode)
                .setReceiveCode(orderInfo.receiveCode)
                .build();
        PaymentManager.getInstance().pay(this, payInfo);
    }

    public static final MediaType JSON
            = MediaType.get("application/json; charset=utf-8");

    /**
     * {
     * "totalAmount":"10",
     * "returnApp":{
     * "activity":"cn.easier.testswitch.JUMP_APP_ACTION",
     * "packageName":"cn.easier.testswitch"
     * },
     * "receiveName":"Test charge",
     * "subject":"Test goods",
     * "appId":"123456",
     * "outTradeNo":"1234569909999999",
     * "notifyUrl":"http://www.baidu.com",
     * "timeoutExpress":"120m",
     * "nonce":"799bcd88b76347b583dac8bc64a4d13d",
     * "shortCode":"100100306",
     * "timestamp":"1657074026539"
     * "timestamp":"1657088151201"timestamp -> {JsonPrimitive@13699} "1657088151201"
     * }
     * 下单
     */
    private void placeOrder() {
        if (TextUtils.isEmpty(orderInfo.amount)){
            Toast.makeText(this, "请输入金额", Toast.LENGTH_SHORT).show();
            return;
        }
        loadingDialog = new LoadingDialog();
        loadingDialog.show(getSupportFragmentManager(), "");
        Log.d(TAG, "placeOrder: ");

        final String outTradeNo = String.valueOf(System.currentTimeMillis());

        final JsonObject returnAppJson = new JsonObject();
        returnAppJson.addProperty("activity", "cn.easier.testswitch.JUMP_APP_ACTION");
        returnAppJson.addProperty("packageName", "cn.easier.testswitch");

        timestamp = String.valueOf(System.currentTimeMillis());
        final String ussd = getUssd(returnAppJson, outTradeNo);

        final String sign = getSign(returnAppJson, outTradeNo);

        final JsonObject jsonObject = new JsonObject();
        jsonObject.addProperty("ussd", ussd);
        jsonObject.addProperty("sign", sign);
        jsonObject.addProperty("appid", Constants.ORDER_APP_ID);
        final String params = jsonObject.toString();

        getOrder(params);
    }

    public static HostnameVerifier getHostnameVerifier() {
        return (hostname, session) -> true;
    }

    /**
     * 下单
     * @param params
     */
    private void getOrder(String params) {
        RequestBody body = RequestBody.create(params, JSON);
        Request request = new Request.Builder()
                .url(BuildConfig.BASE_URL)
                .post(body)
                .build();
        final OkHttpClient.Builder builder = new OkHttpClient.Builder();

        HttpLoggingInterceptor interceptor = new HttpLoggingInterceptor();
        interceptor.setLevel(HttpLoggingInterceptor.Level.BODY);
        builder.addInterceptor(interceptor);
        try {
            SSLContext sslContext = SSLContext.getInstance("TLS");
            X509TrustManager trustManager = new X509TrustManager() {
                @Override
                public void checkClientTrusted(X509Certificate[] chain, String authType) throws CertificateException {

                }

                @Override
                public void checkServerTrusted(X509Certificate[] chain, String authType) throws CertificateException {

                }

                @Override
                public X509Certificate[] getAcceptedIssuers() {
                    return new X509Certificate[0];
                }
            };
            sslContext.init(null, new TrustManager[]{trustManager}, new SecureRandom());
            builder.sslSocketFactory(sslContext.getSocketFactory(), trustManager);
            builder.hostnameVerifier(getHostnameVerifier());
        } catch (Exception e) {
        }
        OkHttpClient client = builder.build();
        client.newCall(request).enqueue(new Callback() {
            @Override
            public void onFailure(@NonNull Call call, @NonNull IOException e) {
                Log.d(TAG, "onFailure: " + e.getMessage());
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        loadingDialog.dismiss();
                        Toast.makeText(MainActivity.this, e.getMessage(), Toast.LENGTH_SHORT).show();
                    }
                });
            }

            @Override
            public void onResponse(@NonNull Call call, @NonNull Response response) throws IOException {
                final ResponseBody body = response.body();

                if (body != null) {
                    BufferedSource source = body.source();
                    source.request(Long.MAX_VALUE);
                    Buffer buffer = source.getBuffer();
                    String string = buffer.clone().readString(StandardCharsets.UTF_8);
                    Log.d(TAG, "onResponse: " + string);
                    if (TextUtils.isEmpty(string)){
                        onFailure(null, new IOException("body is null"));
                        return;
                    }
                    final Order order = new Gson().fromJson(string, Order.class);
                    if (order.success()) {
                        orderInfo.receiveCode = order.getReceiveCode();
                        runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                loadingDialog.dismiss();
                                pay();
                            }
                        });
                    } else {
                        onFailure(null, new IOException(order.msg));
                    }
                } else {
                    onFailure(null, new IOException("body is null"));
                }
            }
        });
    }

    private String getSign(JsonObject returnAppJson, String outTradeNo) {
        String sb = "appId" + "=" + Constants.ORDER_APP_ID +
                "&" + "appKey" + "=" + "c2a7746d7e6346479a8a4fa3dc74bcf7" +
                "&" + "nonce" + "=" + "799bcd88b76347b583dac8bc64a4d13d" +
                "&" + "notifyUrl" + "=" + "http://10.234.55.229:8080/test/querylsl" +
                "&" + "outTradeNo" + "=" + outTradeNo +
                "&" + "receiveName" + "=" + "Test charge" +
                "&" + "returnApp" + "=" + returnAppJson.toString() +
                "&" + "shortCode" + "=" + BuildConfig.SHORT_CODE +
                "&" + "subject" + "=" + "Test goods" +
                "&" + "timeoutExpress" + "=" + "120m" +
                "&" + "timestamp" + "=" + timestamp +
                "&" + "totalAmount" + "=" + orderInfo.amount;
        return EncryptUtils.sign(sb);
    }

    private String getUssd(JsonObject returnAppJson, String outTradeNo) {
        final JsonObject json = new JsonObject();
        json.addProperty("totalAmount", orderInfo.amount);
        json.addProperty("receiveName", "Test charge");
        json.addProperty("subject", "Test goods");
        json.addProperty("appId", Constants.ORDER_APP_ID);
        json.addProperty("outTradeNo", outTradeNo);
        json.addProperty("notifyUrl", "http://10.234.55.229:8080/test/querylsl");
        json.addProperty("timeoutExpress", "120m");
        json.addProperty("nonce", "799bcd88b76347b583dac8bc64a4d13d");
        json.addProperty("shortCode", BuildConfig.SHORT_CODE);
        json.addProperty("timestamp", timestamp);
        json.add("returnApp", returnAppJson);
        return EncryptUtils.encrtpy(json.toString(), Constants.PUBLIC_KEY);
    }
}