/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2020. All rights reserved.
 */

package com.huawei.ethiopia.sdk.utils;

import android.util.Base64;
import android.util.Log;


import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.KeyFactory;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.Signature;
import java.security.SignatureException;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;

import kotlin.text.Charsets;

/**
 * 功能描述：加密算法工具类
 *
 * @author zWX749471
 * @since 2019-07-22
 */
public class EncryptUtils {
    private static final String TAG = "EncryptUtils";
    private static final String RSA = "RSA/ECB/PKCS1Padding";

    public static String rsaEncryptPin(String content) {
        return rsaEncrypt(getPulicKey(Constants.PUBLIC_KEY), content, "RSA");
    }

    /**
     * rsa加密签名
     *
     * @param content 需要被加密的内容
     * @return 返回加密字串
     */
    public static String rsaEncrypt(PublicKey publicKey, String content, String rsa) {
        String encryptString = "";
        try {
            // 使用公钥字符串加密
            if (publicKey == null) {
                throw new NullPointerException("public key is null");
            }
            Cipher cipher = Cipher.getInstance(rsa);
            if (cipher != null) {
                cipher.init(Cipher.ENCRYPT_MODE, publicKey);
                byte[] encryptResults = cipher.doFinal(content.getBytes(Charsets.UTF_8));
                encryptString = Base64.encodeToString(encryptResults, Base64.NO_WRAP);
            }
        } catch (IllegalBlockSizeException e) {
            Log.e(TAG, e.getMessage());
        } catch (BadPaddingException e) {
            Log.e(TAG, e.getMessage());
        } catch (NoSuchAlgorithmException e) {
            Log.e(TAG, e.getMessage());
        } catch (NoSuchPaddingException e) {
            Log.e(TAG, e.getMessage());
        } catch (InvalidKeyException e) {
            Log.e(TAG, e.getMessage());
        }
        return encryptString;
    }

    public static PublicKey getPulicKey(String public_key) {

        try {
            KeyFactory keyFactory = KeyFactory.getInstance("RSA");
            byte[] publicKeyByteArray = Base64.decode(public_key, Base64.DEFAULT);
            X509EncodedKeySpec publickeySpec = new X509EncodedKeySpec(publicKeyByteArray);

            return keyFactory.generatePublic(publickeySpec);
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (InvalidKeySpecException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static String sign(String signStr)  {
        MessageDigest messageDigest = null;
        String sign = null;
        try {
            messageDigest = MessageDigest.getInstance("SHA-256");

            byte[] hashed = messageDigest.digest(signStr.getBytes(StandardCharsets.UTF_8));
            int iterations = Math.max(1, 0) - 1;
            for (int i = 0; i < iterations; ++i) {
                messageDigest.reset();
                hashed = messageDigest.digest(hashed);
            }

            sign = byteToHex(hashed);
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        }
        return sign;
    }

    private static String byteToHex(byte[] data) {
        int l = data.length;
        char[] HEX_DIGITS = new char[]{'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};
        char[] out = new char[l << 1];
        int i = 0;
        for (int var4 = 0; i < l; ++i) {
            out[var4++] = HEX_DIGITS[(240 & data[i]) >>> 4];
            out[var4++] = HEX_DIGITS[15 & data[i]];
        }
        return new String(out);
    }


    public static String sign(String signStr, String private_key) {
        try {
            Signature signature = Signature.getInstance("SHA-256");
            signature.initSign((PrivateKey) getPrivateKey(private_key));
            signature.update(signStr.getBytes(StandardCharsets.UTF_8));

            String signResult = Base64.encodeToString(signature.sign(), Base64.NO_WRAP);
            return signResult.replace("\n", "").replace("\r", "");
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (InvalidKeyException e) {
            e.printStackTrace();
        } catch (SignatureException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static Key getPrivateKey(String private_key) {

        KeyFactory keyFactory = null;
        try {
            keyFactory = KeyFactory.getInstance("RSA");
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        }

        byte[] privateKeyByteArray = Base64.decode(private_key, Base64.DEFAULT);
        PKCS8EncodedKeySpec privatekeySpec = new PKCS8EncodedKeySpec(privateKeyByteArray);
        PrivateKey privateKey = null;
        try {
            privateKey = keyFactory.generatePrivate(privatekeySpec);
        } catch (InvalidKeySpecException e) {
            e.printStackTrace();
        }
        return privateKey;
    }


    public static  String encrtpy(String context,String public_key) {
        try {
            byte[] bytes = context.getBytes(StandardCharsets.UTF_8);
            StringBuilder stringBuilder = new StringBuilder();

            int MAX_DATA_LENGTH = 117;

            Cipher  cipher = Cipher.getInstance(RSA);

            //公钥加密
            cipher.init(Cipher.ENCRYPT_MODE,getPulicKey(public_key));

            int offSet = 0;
            int i = 0;
            int inputLength = bytes.length;
            while (inputLength - offSet > 0){
                if (inputLength - offSet > MAX_DATA_LENGTH){
                    stringBuilder.append(Base64.encodeToString(cipher.doFinal(bytes,offSet,MAX_DATA_LENGTH), Base64.NO_WRAP ).replace("\r","").replace("\n",""));
                }else{
                    stringBuilder.append(Base64.encodeToString(cipher.doFinal(bytes,offSet,inputLength - offSet), Base64.NO_WRAP).replace("\r","").replace("\n",""));
                }
                i++;
                offSet = i*MAX_DATA_LENGTH;
            }
            return  stringBuilder.toString();
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (NoSuchPaddingException e) {
            e.printStackTrace();
        } catch (InvalidKeyException e) {
            e.printStackTrace();
        } catch (BadPaddingException e) {
            e.printStackTrace();
        } catch (IllegalBlockSizeException e) {
            e.printStackTrace();
        }

        return null;

    }
}
