package com.huawei.ethiopia.sdk.widget;

import android.animation.ObjectAnimator;
import android.content.Context;
import android.util.AttributeSet;
import android.view.animation.LinearInterpolator;
import android.widget.ImageView;

import androidx.annotation.Nullable;

/**
 * 旋转动画ImageView
 *
 * @author s30027976
 * @since 2022/6/16
 */
public class RotateImageView extends ImageView {
    private static final String TAG = "RotateImageView";
    private static final int ANIMATOR_DURATION = 3000;
    private ObjectAnimator anim;

    public RotateImageView(Context context) {
        super(context);
    }

    public RotateImageView(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
    }

    public RotateImageView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
    }

    @Override
    protected void onAttachedToWindow() {
        super.onAttachedToWindow();
        startAnim();
    }

    @Override
    protected void onDetachedFromWindow() {
        super.onDetachedFromWindow();
        endAnim();
    }

    private void endAnim() {
        if (anim != null && anim.isRunning()) {
            anim.pause();
        }
    }

    private void startAnim() {
        if (anim == null) {
            anim = ObjectAnimator.ofFloat(this, "rotation", 0f, 360f);
            anim.setDuration(ANIMATOR_DURATION); // 设置动画时间
            anim.setInterpolator(new LinearInterpolator()); // 动画时间线性渐变
            anim.setRepeatCount(ObjectAnimator.INFINITE);
            anim.setRepeatMode(ObjectAnimator.RESTART);
        }
        anim.start();
    }
}
